package com.sun.naming.internal;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Iterator;
import javax.naming.NamingEnumeration;
import java.io.InputStream;
import java.io.IOException;
import javax.naming.ConfigurationException;
import java.util.Properties;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.util.HashMap;
import javax.naming.Context;
import javax.naming.NamingException;
import java.lang.ref.WeakReference;
import java.util.List;
import java.util.Map;
import java.util.Hashtable;
import java.util.WeakHashMap;

public final class ResourceManager
{
    private static final String PROVIDER_RESOURCE_FILE_NAME = "jndiprovider.properties";
    private static final String APP_RESOURCE_FILE_NAME = "jndi.properties";
    private static final String JRELIB_PROPERTY_FILE_NAME = "jndi.properties";
    private static final String DISABLE_APP_RESOURCE_FILES = "com.sun.naming.disable.app.resource.files";
    private static final String[] listProperties;
    private static final VersionHelper helper;
    private static final WeakHashMap<Object, Hashtable<? super String, Object>> propertiesCache;
    private static final WeakHashMap<ClassLoader, Map<String, List<NamedWeakReference<Object>>>> factoryCache;
    private static final WeakHashMap<ClassLoader, Map<String, WeakReference<Object>>> urlFactoryCache;
    private static final WeakReference<Object> NO_FACTORY;
    
    private ResourceManager() {
    }
    
    public static Hashtable<?, ?> getInitialEnvironment(Hashtable<?, ?> hashtable) throws NamingException {
        String[] props = VersionHelper.PROPS;
        if (hashtable == null) {
            hashtable = new Hashtable(11);
        }
        Object value = hashtable.get("java.naming.applet");
        String[] jndiProperties = ResourceManager.helper.getJndiProperties();
        for (int i = 0; i < props.length; ++i) {
            String value2 = hashtable.get(props[i]);
            if (value2 == null) {
                if (value != null) {
                    value2 = ResourceManager.AppletParameter.get(value, props[i]);
                }
                if (value2 == null) {
                    value2 = ((jndiProperties != null) ? jndiProperties[i] : ResourceManager.helper.getJndiProperty(i));
                }
                if (value2 != null) {
                    hashtable.put(props[i], value2);
                }
            }
        }
        String s = (String)hashtable.get("com.sun.naming.disable.app.resource.files");
        if (s != null && s.equalsIgnoreCase("true")) {
            return hashtable;
        }
        mergeTables(hashtable, getApplicationResources());
        return hashtable;
    }
    
    public static String getProperty(String s, Hashtable<?, ?> hashtable, Context context, boolean b) throws NamingException {
        String str = (hashtable != null) ? ((String)hashtable.get(s)) : null;
        if (context == null || (str != null && !b)) {
            return str;
        }
        String str2 = (String)getProviderResource(context).get(s);
        if (str == null) {
            return str2;
        }
        if (str2 == null || !b) {
            return str;
        }
        return str + ":" + str2;
    }
    
    public static FactoryEnumeration getFactories(String s, Hashtable<?, ?> hashtable, Context context) throws NamingException {
        String property = getProperty(s, hashtable, context, true);
        if (property == null) {
            return null;
        }
        ClassLoader contextClassLoader = ResourceManager.helper.getContextClassLoader();
        Map<String, ArrayList> value = null;
        synchronized (ResourceManager.factoryCache) {
            value = (Map)ResourceManager.factoryCache.get(contextClassLoader);
            if (value == null) {
                value = new HashMap(11);
                ResourceManager.factoryCache.put(contextClassLoader, value);
            }
        }
        synchronized (value) {
            List<Object> list = (List)value.get(property);
            if (list != null) {
                return (list.size() == 0) ? null : new FactoryEnumeration(list, contextClassLoader);
            }
            StringTokenizer stringTokenizer = new StringTokenizer(property, ":");
            ArrayList list2 = new ArrayList(5);
            while (stringTokenizer.hasMoreTokens()) {
                try {
                    String nextToken = stringTokenizer.nextToken();
                    list2.add(new NamedWeakReference(ResourceManager.helper.loadClass(nextToken, contextClassLoader), nextToken));
                }
                catch (Exception ex) {}
            }
            value.put(property, list2);
            return new FactoryEnumeration(list2, contextClassLoader);
        }
    }
    
    public static Object getFactory(String s, Hashtable<?, ?> hashtable, Context context, String s2, String str) throws NamingException {
        String property = getProperty(s, hashtable, context, true);
        String string;
        if (property != null) {
            string = property + ":" + str;
        }
        else {
            string = str;
        }
        ClassLoader contextClassLoader = ResourceManager.helper.getContextClassLoader();
        String string2 = s2 + " " + string;
        Object value = null;
        synchronized (ResourceManager.urlFactoryCache) {
            value = (Map<String, WeakReference<Object>>)ResourceManager.urlFactoryCache.get(contextClassLoader);
            if (value == null) {
                value = new HashMap(11);
                ResourceManager.urlFactoryCache.put(contextClassLoader, value);
            }
        }
        synchronized (value) {
            Object referent = null;
            WeakReference<Object> weakReference = (WeakReference)value.get(string2);
            if (weakReference == ResourceManager.NO_FACTORY) {
                return null;
            }
            if (weakReference != null) {
                referent = weakReference.get();
                if (referent != null) {
                    return referent;
                }
            }
            StringTokenizer o2 = new StringTokenizer(string, ":");
            while (referent == null && o2.hasMoreTokens()) {
                String string3 = o2.nextToken() + s2;
                try {
                    referent = ResourceManager.helper.loadClass(string3, contextClassLoader).newInstance();
                }
                catch (InstantiationException rootCause) {
                    NamingException o3 = new NamingException("Cannot instantiate " + string3);
                    o3.setRootCause(rootCause);
                    throw o3;
                }
                catch (IllegalAccessException rootCause2) {
                    NamingException o4 = new NamingException("Cannot access " + string3);
                    o4.setRootCause(rootCause2);
                    throw o4;
                }
                catch (Exception ex) {}
            }
            value.put(string2, (referent != null) ? new WeakReference(referent) : ResourceManager.NO_FACTORY);
            return referent;
        }
    }
    
    private static Hashtable<? super String, Object> getProviderResource(Object o) throws NamingException {
        if (o == null) {
            return new Hashtable(1);
        }
        synchronized (ResourceManager.propertiesCache) {
            Class<Object> class1 = o.getClass();
            Hashtable<Object, Object> hashtable = (Hashtable)ResourceManager.propertiesCache.get(class1);
            if (hashtable != null) {
                return hashtable;
            }
            Properties value = new Properties();
            InputStream resourceAsStream = ResourceManager.helper.getResourceAsStream(class1, "jndiprovider.properties");
            if (resourceAsStream != null) {
                try {
                    ((Properties)value).load(resourceAsStream);
                }
                catch (IOException rootCause) {
                    ConfigurationException ex = new ConfigurationException("Error reading provider resource file for " + class1);
                    ex.setRootCause(rootCause);
                    throw ex;
                }
            }
            ResourceManager.propertiesCache.put(class1, value);
            return value;
        }
    }
    
    private static Hashtable<? super String, Object> getApplicationResources() throws NamingException {
        ClassLoader contextClassLoader = ResourceManager.helper.getContextClassLoader();
        synchronized (ResourceManager.propertiesCache) {
            Object value = (Hashtable<? super String, Object>)ResourceManager.propertiesCache.get(contextClassLoader);
            if (value != null) {
                return value;
            }
            try {
                NamingEnumeration<InputStream> resources = ResourceManager.helper.getResources(contextClassLoader, "jndi.properties");
                try {
                    while (resources.hasMore()) {
                        Properties o = new Properties();
                        InputStream inStream = (InputStream)resources.next();
                        try {
                            o.load(inStream);
                        }
                        finally {
                            inStream.close();
                        }
                        if (value == null) {
                            value = o;
                        }
                        else {
                            mergeTables(value, o);
                        }
                    }
                }
                finally {
                    while (resources.hasMore()) {
                        ((InputStream)resources.next()).close();
                    }
                }
                InputStream javaHomeLibStream = ResourceManager.helper.getJavaHomeLibStream("jndi.properties");
                if (javaHomeLibStream != null) {
                    try {
                        Properties o2 = new Properties();
                        o2.load(javaHomeLibStream);
                        if (value == null) {
                            value = o2;
                        }
                        else {
                            mergeTables(value, o2);
                        }
                    }
                    finally {
                        javaHomeLibStream.close();
                    }
                }
            }
            catch (IOException rootCause) {
                ConfigurationException o3 = new ConfigurationException("Error reading application resource file");
                o3.setRootCause(rootCause);
                throw o3;
            }
            if (value == null) {
                value = new Hashtable(11);
            }
            ResourceManager.propertiesCache.put(contextClassLoader, value);
            return value;
        }
    }
    
    private static void mergeTables(Hashtable<? super String, Object> hashtable, Hashtable<? super String, Object> hashtable2) {
        for (String key : hashtable2.keySet()) {
            Object value = hashtable.get(key);
            if (value == null) {
                hashtable.put(key, hashtable2.get(key));
            }
            else if (isListProperty(key)) {
                hashtable.put(key, (String)value + ":" + (String)hashtable2.get(key));
            }
            else {
                continue;
            }
        }
    }
    
    private static boolean isListProperty(String intern) {
        intern = intern.intern();
        int i = 0;
        while (i < ResourceManager.listProperties.length) {
            if (intern == ResourceManager.listProperties[i]) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    static {
        listProperties = new String[] { "java.naming.factory.object", "java.naming.factory.url.pkgs", "java.naming.factory.state", "java.naming.factory.control" };
        helper = VersionHelper.getVersionHelper();
        propertiesCache = new WeakHashMap(11);
        factoryCache = new WeakHashMap(11);
        urlFactoryCache = new WeakHashMap(11);
        NO_FACTORY = new WeakReference(null);
    }
    
    private static class AppletParameter
    {
        private static final Class<?> clazz;
        private static final Method getMethod;
        
        private static Class<?> getClass(String name) {
            try {
                return Class.forName(name, true, null);
            }
            catch (ClassNotFoundException ex) {
                return null;
            }
        }
        
        private static Method getMethod(Class<?> clazz, String name, Class<?>... parameterTypes) {
            if (clazz != null) {
                try {
                    return clazz.getMethod(name, parameterTypes);
                }
                catch (NoSuchMethodException detailMessage) {
                    throw new AssertionError(detailMessage);
                }
            }
            else {
                return null;
            }
        }
        
        static Object get(Object obj, String s) {
            if (ResourceManager.AppletParameter.clazz == null || !ResourceManager.AppletParameter.clazz.isInstance(obj)) {
                throw new ClassCastException(obj.getClass().getName());
            }
            try {
                return ResourceManager.AppletParameter.getMethod.invoke(obj, new Object[] { s });
            }
            catch (InvocationTargetException | IllegalAccessException detailMessage) {
                throw new AssertionError(detailMessage);
            }
        }
        
        static {
            clazz = getClass("java.applet.Applet");
            getMethod = getMethod(ResourceManager.AppletParameter.clazz, "getParameter", new Class[] { String.class });
        }
    }
}
